@@ -17,6 +17,8 @@ import (
 	"time"
 
 	"github.com/quic-go/quic-go"
+	"github.com/quic-go/quic-go/logging"
+	"github.com/quic-go/quic-go/qlog"
 )
 
 // MaxIdleTimeoutForQuicConnections for quic connections. The default is 30 which we have replicated here.
@@ -90,6 +92,7 @@ func (s *Netceptor) listen(ctx context.Context, service string, tlscfg *tls.Conf
 	s.Logger.Debug("%s added service %s to listener registry", s.nodeID, service)
 	s.listenerRegistry[service] = pc
 	cfg := &quic.Config{
+		Tracer:                  s.tracer,
 		HandshakeIdleTimeout:    15 * time.Second,
 		MaxIdleTimeout:          MaxIdleTimeoutForQuicConnections,
 		Allow0RTT:               true,
@@ -134,6 +137,27 @@ func (s *Netceptor) listen(ctx context.Context, service string, tlscfg *tls.Conf
 	return li, nil
 }
 
+func (s *Netceptor) tracer(ctx context.Context, p logging.Perspective, connID quic.ConnectionID) *logging.ConnectionTracer {
+	qlogPath := os.Getenv("QLOGDIR")
+	if qlogPath != "" {
+		role := "server"
+		if p == logging.PerspectiveClient {
+			role = "client"
+		}
+		filename := fmt.Sprintf("log_%x_%s.qlog", connID, role)
+		f, err := os.Create(qlogPath + filename)
+		if err != nil {
+			s.Logger.Debug("failed to create qlog file at path: %s", qlogPath)
+
+			return nil
+		}
+
+		return qlog.NewConnectionTracer(f, p, connID)
+	} else {
+		return nil
+	}
+}
+
 // Listen returns a stream listener compatible with Go's net.Listener.
 // If service is blank, generates and uses an ephemeral service name.
 func (s *Netceptor) Listen(service string, tlscfg *tls.Config) (*Listener, error) {
@@ -145,8 +169,10 @@ func (s *Netceptor) ListenAndAdvertise(service string, tlscfg *tls.Config, tags
 	return s.listen(s.context, service, tlscfg, true, tags)
 }
 
-func (li *Listener) sendResult(conn net.Conn, err error) {
+func (li *Listener) sendResult(ctx context.Context, conn net.Conn, err error) {
 	select {
+	case <-ctx.Done():
+		return
 	case li.acceptChan <- &acceptResult{
 		conn: conn,
 		err:  err,
@@ -171,7 +197,7 @@ func (li *Listener) acceptLoop(ctx context.Context) {
 		default:
 		}
 		if err != nil {
-			li.sendResult(nil, err)
+			li.sendResult(ctx, nil, err)
 
 			continue
 		}
@@ -192,7 +218,7 @@ func (li *Listener) acceptLoop(ctx context.Context) {
 				return
 			} else if err != nil {
 				_ = qc.CloseWithError(500, fmt.Sprintf("AcceptStream Error: %s", err.Error()))
-				li.sendResult(nil, err)
+				li.sendResult(ctx, nil, err)
 
 				return
 			}
@@ -200,13 +226,13 @@ func (li *Listener) acceptLoop(ctx context.Context) {
 			n, err := qs.Read(buf)
 			if err != nil {
 				_ = qc.CloseWithError(500, fmt.Sprintf("Read Error: %s", err.Error()))
-				li.sendResult(nil, err)
+				li.sendResult(ctx, nil, err)
 
 				return
 			}
 			if n != 1 || buf[0] != 0 {
 				_ = qc.CloseWithError(500, "Read Data Error")
-				li.sendResult(nil, fmt.Errorf("stream failed to initialize"))
+				li.sendResult(ctx, nil, fmt.Errorf("stream failed to initialize"))
 
 				return
 			}
@@ -235,7 +261,7 @@ func (li *Listener) acceptLoop(ctx context.Context) {
 					return
 				}
 			}()
-			li.sendResult(conn, err)
+			li.sendResult(ctx, conn, err)
 		}()
 	}
 }
@@ -294,6 +320,7 @@ func (s *Netceptor) DialContext(ctx context.Context, node string, service string
 	}
 	rAddr := s.NewAddr(node, service)
 	cfg := &quic.Config{
+		Tracer:                  s.tracer,
 		HandshakeIdleTimeout:    15 * time.Second,
 		MaxIdleTimeout:          MaxIdleTimeoutForQuicConnections,
 		Allow0RTT:               true,
