@@ -171,7 +171,7 @@ func (ku KubeAPIWrapper) NewFakeAlwaysRateLimiter() flowcontrol.RateLimiter {
 // It is instantiated in the NewkubeWorker function and available throughout the package.
 var KubeAPIWrapperInstance KubeAPIer
 
-var KubeAPIWrapperLock *sync.RWMutex
+var KubeAPIWrapperLock sync.Mutex
 
 // ErrPodCompleted is returned when pod has already completed before we could attach.
 var ErrPodCompleted = fmt.Errorf("pod ran to completion")
@@ -229,6 +229,25 @@ func podRunningAndReady() func(event watch.Event) (bool, error) {
 	return inner
 }
 
+func GetTimeoutOpenLogstream(kw *KubeUnit) int {
+	// RECEPTOR_OPEN_LOGSTREAM_TIMEOUT
+	// default: 1
+	openLogStreamTimeout := 1
+	envTimeout := os.Getenv("RECEPTOR_OPEN_LOGSTREAM_TIMEOUT")
+	if envTimeout != "" {
+		var err error
+		openLogStreamTimeout, err = strconv.Atoi(envTimeout)
+		if err != nil || openLogStreamTimeout < 1 {
+			// ignore error, use default
+			kw.GetWorkceptor().nc.GetLogger().Warning("Invalid value for RECEPTOR_OPEN_LOGSTREAM_TIMEOUT: %s. Ignoring", envTimeout)
+			openLogStreamTimeout = 1
+		}
+	}
+	kw.GetWorkceptor().nc.GetLogger().Debug("RECEPTOR_OPEN_LOGSTREAM_TIMEOUT: %d", openLogStreamTimeout)
+
+	return openLogStreamTimeout
+}
+
 func (kw *KubeUnit) kubeLoggingConnectionHandler(timestamps bool, sinceTime time.Time) (io.ReadCloser, error) {
 	var logStream io.ReadCloser
 	var err error
@@ -257,7 +276,7 @@ func (kw *KubeUnit) kubeLoggingConnectionHandler(timestamps bool, sinceTime time
 			retries,
 			err,
 		)
-		time.Sleep(time.Second)
+		time.Sleep(time.Duration(GetTimeoutOpenLogstream(kw)) * time.Second)
 	}
 	if err != nil {
 		errMsg := fmt.Sprintf("Error opening log stream for pod %s/%s. Error: %s", podNamespace, podName, err)
@@ -294,7 +313,7 @@ func (kw *KubeUnit) kubeLoggingNoReconnect(streamWait *sync.WaitGroup, stdout *S
 	}
 }
 
-func (kw *KubeUnit) kubeLoggingWithReconnect(streamWait *sync.WaitGroup, stdout *STDoutWriter, stdinErr *error, stdoutErr *error) {
+func (kw *KubeUnit) KubeLoggingWithReconnect(streamWait *sync.WaitGroup, stdout *STDoutWriter, stdinErr *error, stdoutErr *error) {
 	// preferred method for k8s >= 1.23.14
 	defer streamWait.Done()
 	var sinceTime time.Time
@@ -377,8 +396,17 @@ func (kw *KubeUnit) kubeLoggingWithReconnect(streamWait *sync.WaitGroup, stdout
 				// At this point we exausted all retries, every retry we either failed to read OR we read but did not get newer msg
 				// If we got a EOF on the last retry we assume that we read everything and we can stop the loop
 				// we ASSUME this is the happy path.
+				// If kube api returned an error there is a missing new line and that line never gets read.
 				if err != io.EOF {
 					*stdoutErr = err
+				} else if line != "" && err == io.EOF {
+					_, err = stdout.Write([]byte(line + "\n"))
+					if err != nil {
+						*stdoutErr = fmt.Errorf("writing to stdout: %s", err)
+						kw.GetWorkceptor().nc.GetLogger().Error("Error writing to stdout: %s", err)
+
+						return
+					}
 				}
 
 				return
@@ -411,7 +439,7 @@ func (kw *KubeUnit) kubeLoggingWithReconnect(streamWait *sync.WaitGroup, stdout
 	}
 }
 
-func (kw *KubeUnit) createPod(env map[string]string) error {
+func (kw *KubeUnit) CreatePod(env map[string]string) error {
 	ked := kw.UnredactedStatus().ExtraData.(*KubeExtraData)
 	command, err := shlex.Split(ked.Command)
 	if err != nil {
@@ -513,7 +541,9 @@ func (kw *KubeUnit) createPod(env map[string]string) error {
 	})
 
 	// Wait for the pod to be running
+	KubeAPIWrapperLock.Lock()
 	fieldSelector := KubeAPIWrapperInstance.OneTermEqualSelector("metadata.name", kw.pod.Name).String()
+	KubeAPIWrapperLock.Unlock()
 	lw := &cache.ListWatch{
 		ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 			options.FieldSelector = fieldSelector
@@ -535,7 +565,9 @@ func (kw *KubeUnit) createPod(env map[string]string) error {
 	}
 
 	time.Sleep(2 * time.Second)
+	KubeAPIWrapperLock.Lock()
 	ev, err := KubeAPIWrapperInstance.UntilWithSync(ctxPodReady, lw, &corev1.Pod{}, nil, podRunningAndReady())
+	KubeAPIWrapperLock.Unlock()
 	if ev == nil || ev.Object == nil {
 		return fmt.Errorf("did not return an event while watching pod for work unit %s", kw.ID())
 	}
@@ -612,7 +644,7 @@ func (kw *KubeUnit) runWorkUsingLogger() {
 	if podName == "" {
 		// create new pod if ked.PodName is empty
 		// TODO: add retry logic to make this more resilient to transient errors
-		if err := kw.createPod(nil); err != nil {
+		if err := kw.CreatePod(nil); err != nil {
 			if err != ErrPodCompleted {
 				errMsg := fmt.Sprintf("Error creating pod: %s", err)
 				kw.GetWorkceptor().nc.GetLogger().Error(errMsg) //nolint:govet
@@ -833,7 +865,7 @@ func (kw *KubeUnit) runWorkUsingLogger() {
 	stdoutWithReconnect := ShouldUseReconnect(kw)
 	if stdoutWithReconnect && stdoutErr == nil {
 		kw.GetWorkceptor().nc.GetLogger().Debug("streaming stdout with reconnect support")
-		go kw.kubeLoggingWithReconnect(&streamWait, stdout, &stdinErr, &stdoutErr)
+		go kw.KubeLoggingWithReconnect(&streamWait, stdout, &stdinErr, &stdoutErr)
 	} else {
 		kw.GetWorkceptor().nc.GetLogger().Debug("streaming stdout with no reconnect support")
 		go kw.kubeLoggingNoReconnect(&streamWait, stdout, &stdoutErr)
@@ -1053,7 +1085,7 @@ func (kw *KubeUnit) runWorkUsingTCP() {
 	}()
 
 	// Create the pod
-	err = kw.createPod(map[string]string{"RECEPTOR_HOST": listenHost, "RECEPTOR_PORT": listenPort})
+	err = kw.CreatePod(map[string]string{"RECEPTOR_HOST": listenHost, "RECEPTOR_PORT": listenPort})
 	if err != nil {
 		errMsg := fmt.Sprintf("Error creating pod: %s", err)
 		kw.UpdateBasicStatus(WorkStateFailed, errMsg, 0)
@@ -1565,11 +1597,11 @@ func (cfg KubeWorkerCfg) NewkubeWorker(bwu BaseWorkUnitForWorkUnit, w *Workcepto
 		}
 	}
 
-	KubeAPIWrapperLock = &sync.RWMutex{}
 	KubeAPIWrapperLock.Lock()
-	KubeAPIWrapperInstance = KubeAPIWrapper{}
 	if kawi != nil {
 		KubeAPIWrapperInstance = kawi
+	} else {
+		KubeAPIWrapperInstance = KubeAPIWrapper{}
 	}
 	KubeAPIWrapperLock.Unlock()
 
