@@ -37,6 +37,7 @@ const (
 type WatcherWrapper interface {
 	Add(name string) error
 	Close() error
+	ErrorChannel() chan error
 	EventChannel() chan fsnotify.Event
 }
 
@@ -52,6 +53,10 @@ func (rw *RealWatcher) Close() error {
 	return rw.watcher.Close()
 }
 
+func (rw *RealWatcher) ErrorChannel() chan error {
+	return rw.watcher.Errors
+}
+
 func (rw *RealWatcher) EventChannel() chan fsnotify.Event {
 	return rw.watcher.Events
 }
@@ -126,6 +131,7 @@ func (bwu *BaseWorkUnit) Init(w *Workceptor, unitID string, workType string, fs
 		if err == nil {
 			bwu.watcher = &RealWatcher{watcher: watcher}
 		} else {
+			bwu.w.nc.GetLogger().Info("fsnotify.NewWatcher returned %s", err)
 			bwu.watcher = nil
 		}
 	}
@@ -223,7 +229,11 @@ func (sfd *StatusFileData) Save(filename string) error {
 	}
 	err = sfd.saveToFile(file)
 	if err != nil {
-		_ = file.Close()
+		serr := file.Close()
+
+		if serr != nil {
+			MainInstance.nc.GetLogger().Error("Error closing %s: %s", filename, serr)
+		}
 
 		return err
 	}
@@ -262,7 +272,10 @@ func (sfd *StatusFileData) Load(filename string) error {
 	}
 	err = sfd.loadFromFile(file)
 	if err != nil {
-		_ = file.Close()
+		lerr := file.Close()
+		if lerr != nil {
+			MainInstance.nc.GetLogger().Error("Error closing %s: %s", filename, lerr)
+		}
 
 		return err
 	}
@@ -385,20 +398,31 @@ func (bwu *BaseWorkUnit) MonitorLocalStatus() {
 	var watcherEvents chan fsnotify.Event
 	watcherEvents = make(chan fsnotify.Event)
 
+	var watcherErrors chan error
+	watcherErrors = make(chan error)
+
 	if bwu.watcher != nil {
 		err := bwu.watcher.Add(statusFile)
 		if err == nil {
 			defer func() {
-				_ = bwu.watcher.Close()
+				werr := bwu.watcher.Close()
+				if werr != nil {
+					bwu.w.nc.GetLogger().Error("Error in defer closing %s: %s", statusFile, err)
+				}
 			}()
 			watcherEvents = bwu.watcher.EventChannel()
+			watcherErrors = bwu.watcher.ErrorChannel()
 		} else {
-			_ = bwu.watcher.Close()
+			werr := bwu.watcher.Close()
+			if werr != nil {
+				bwu.w.nc.GetLogger().Error("Error closing %s: %s", statusFile, err)
+			}
 			bwu.watcher = nil
 		}
 	}
 	fi, err := bwu.fs.Stat(statusFile)
 	if err != nil {
+		bwu.w.nc.GetLogger().Error("Error retrieving stat for %s: %s", statusFile, err)
 		fi = nil
 	}
 
@@ -408,10 +432,23 @@ loop:
 		case <-bwu.ctx.Done():
 			break loop
 		case event := <-watcherEvents:
-			if event.Op&fsnotify.Write == fsnotify.Write {
+			switch {
+			case event.Has(fsnotify.Create):
+				bwu.w.nc.GetLogger().Debug("Watcher Event create of %s", statusFile)
+			case event.Op&fsnotify.Write == fsnotify.Write:
 				err = bwu.Load()
 				if err != nil {
-					bwu.w.nc.GetLogger().Error("Error reading %s: %s", statusFile, err)
+					bwu.w.nc.GetLogger().Error("Watcher Events Error reading %s: %s", statusFile, err)
+				}
+			case event.Op&fsnotify.Remove == fsnotify.Remove:
+				err = bwu.Load()
+				if err != nil {
+					bwu.w.nc.GetLogger().Debug("Watcher Events Remove reading %s: %s", statusFile, err)
+				}
+			case event.Op&fsnotify.Rename == fsnotify.Rename:
+				err = bwu.Load()
+				if err != nil {
+					bwu.w.nc.GetLogger().Debug("Watcher Events Rename reading %s: %s", statusFile, err)
 				}
 			}
 		case <-time.After(time.Second):
@@ -420,9 +457,14 @@ loop:
 				fi = newFi
 				err = bwu.Load()
 				if err != nil {
-					bwu.w.nc.GetLogger().Error("Error reading %s: %s", statusFile, err)
+					bwu.w.nc.GetLogger().Error("Work unit load Error reading %s: %s", statusFile, err)
 				}
 			}
+		case err, ok := <-watcherErrors:
+			if !ok {
+				return
+			}
+			bwu.w.nc.GetLogger().Error("fsnotify Error reading %s: %s", statusFile, err)
 		}
 		complete := IsComplete(bwu.Status().State)
 		if complete {
